/*
 * @Author: LiZhiWei
 * @Date: 2025-04-15 15:24:14
 * @LastEditors: LiZhiWei
 * @LastEditTime: 2025-04-15 15:28:58
 * @Description: 
 */
import axios from 'axios'
import type { AxiosRequestConfig, AxiosInstance, AxiosResponse } from 'axios'
import { Message } from 'element-ui'

class HttpRequest {
  private baseUrl: string

  constructor (baseUrl: string) {
    this.baseUrl = baseUrl
  }

  private getInsideConfig () {
    const config = {
      baseURL: this.baseUrl,
      timeout: 60000,
      withCredentials: true,
      headers: {
        Accept: 'application/json;charset=utf-8'
      }
    }
    return config
  }

  private interceptors (instance: AxiosInstance) {
    // 请求拦截
    instance.interceptors.request.use(
      (config: AxiosRequestConfig) => {
        return config
      },
      error => {
        return Promise.reject(error)
      }
    )

    // 响应拦截
    instance.interceptors.response.use(
      (response: AxiosResponse) => {
        return response
      },
      error => {
        Message.error(error.response?.data?.message || '请求失败')
        return Promise.reject(error)
      }
    )
  }

  request<T = any> (options: any): Promise<T> {
    const instance: AxiosInstance = axios.create()
    options = Object.assign(this.getInsideConfig(), options)
    this.interceptors(instance)
    return instance(options) as unknown as Promise<T>
  }
}

export default HttpRequest